import os, sys 

from auto_machinelearning.ml.regression.components.data_ingestion import DataIngestion
from auto_machinelearning.ml.regression.components.data_transformation import DataTransformation
from auto_machinelearning.ml.regression.components.model_trainer import ModelTrainer


class AutoRegression: 
    def __init__(self, path: str, filetype: str, target_column: str, **params):
        self.path = path 
        self.filetype = filetype 
        self.target_column = target_column
        self.params = params
        
    def train_model(self): 
        self.data_ingestion = DataIngestion(path=self.path, file_type=self.filetype, **self.params)
        self.data_ingestion_artifacts = self.data_ingestion.run_data_ingestion()

        self.data_transformation = DataTransformation(self.data_ingestion_artifacts, target_column=self.target_column)
        self.data_transformation_artifacts = self.data_transformation.run_data_transformation()

        self.model_trainer = ModelTrainer(self.data_transformation_artifacts)
        self.model_trainer_artifacts = self.model_trainer.train_model()

        return {
            "best_model_name": self.model_trainer_artifacts.best_model_name, 
            "model": self.model_trainer_artifacts.trained_model, 
            "model_params": self.model_trainer_artifacts.best_model_parameters, 
            "r2_score": self.model_trainer_artifacts.train_metric_artifact.r2_score, 
            "rmse": self.model_trainer_artifacts.train_metric_artifact.rmse, 
            "mse": self.model_trainer_artifacts.train_metric_artifact.mse 
            # "models_report":self.model_trainer_artifacts.models_report 
        }


import os, sys 
import pandas as pd 
import numpy as np 

from sklearn.model_selection import train_test_split

from auto_machinelearning.ml.preprocessor.datareader import ReadData
from auto_machinelearning.ml.classification.entity.artifacts_entity import DataIngestionArtifacts

class DataIngestion: 
    def __init__(self, path: str, file_type: str, **params): 
        self.path = path 
        self.file_type = file_type
        self.params = params
        self.df: pd.DataFrame | None = None
        self.train_df: pd.DataFrame | None = None
        self.test_df: pd.DataFrame | None = None    

    def data_ingestion(self) -> pd.DataFrame: 
        data_reader = ReadData()
        if self.file_type == "csv": 
            self.df = data_reader.read_csv(self.path, **self.params)
        elif self.file_type == "excel": 
            self.df = data_reader.read_excel(self.path, **self.params)
        else:
            raise ValueError(f"Unsupported file type: {self.file_type}")
        return self.df

    def data_train_test_split(self) -> tuple[pd.DataFrame, pd.DataFrame]:
        train, test = train_test_split(self.df, test_size=0.2, random_state=42) 
        return train, test

    def run_data_ingestion(self) -> DataIngestionArtifacts: 
        self.df = self.data_ingestion()
        self.train_df, self.test_df = self.data_train_test_split()
        return DataIngestionArtifacts(
            df=self.df, 
            train_df=self.train_df, 
            test_df=self.test_df 
        )
        



import os, sys 
import joblib 
import numpy as np 
import pandas as pd 

from auto_machinelearning.ml.preprocessor.datareader import ReadData
from auto_machinelearning.ml.preprocessor.datadescriber import DataDescribe
from auto_machinelearning.ml.preprocessor.auto_preprocess import AutoPreprocessor
from auto_machinelearning.ml.preprocessor.processor import Preprocess


class AutoPreprocess: 
    def __init__(self, path: str, file_type: str, store_path: str, preprocessor_store_path: str,  target_column: str, **params): 
        self.path = path
        self.file_type = file_type
        self.store_path = store_path
        self.target_column = target_column
        self.preprocessor_store_path = preprocessor_store_path
        self.params = params
        self.df = None
        self.description = None
        self.preprocessed_df = None
        self.encoder = None
        self.scaler = None

    def read_file(self):
        read_data = ReadData()
        if self.file_type == "csv": 
            self.df = read_data.read_csv(self.path, **self.params)
        elif self.file_type == "excel": 
            self.df = read_data.read_excel(self.path, **self.params)
        return self.df
    
    def describe_data(self, df: pd.DataFrame, target_column: str):
        data_describer = DataDescribe(df, target_column)
        data_describer.summarize()
        description = data_describer.get_summary_dict()
        return description

    def preprocess_data(self): 
        data_processor = Preprocess(self.df, self.target_column, self.store_path, self.preprocessor_store_path)
        self.preprocessed_df =  data_processor.fit()
        return self.preprocessed_df

    def process(self): 
        self.df = self.read_file()
        self.description = self.describe_data(self.df, self.target_column)
        self.preprocessed_df = self.preprocess_data()
        return self.preprocessed_df



import os 
import sys 
import pandas as pd 
import numpy as np 
import logging

from sklearn.linear_model import LinearRegression
from sklearn.linear_model import Lasso, Ridge
from sklearn.tree import DecisionTreeRegressor
from sklearn.svm import SVR
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
from sklearn.metrics import mean_squared_error, r2_score

from auto_machinelearning.ml.regression.utils import load_model_params

from auto_machinelearning.ml.regression.entity.artifacts_entity import RegressionMetrixArtifacts, ModelTrainerArtifact, DataTransformationArtifacts


class ModelTrainer: 
    def __init__(self, data_transformation_artifacts: DataTransformationArtifacts): 
        self.data_transformation_artifacts = data_transformation_artifacts
        self.X_train = self.data_transformation_artifacts.X_train
        self.X_test = self.data_transformation_artifacts.X_test
        self.y_train = self.data_transformation_artifacts.y_train
        self.y_test = self.data_transformation_artifacts.y_test


    def evaluate_models(self, X_train, y_train, X_test, y_test, models, params): 
        report = {}
        tuned_models = {}  # NEW: to store best estimators

        for i in range(len(list(models))): 
            model_name = list(models.keys())[i]
            model = list(models.values())[i]
            para = params.get(model_name, {})

            # gs = GridSearchCV(model, para, cv=3, n_jobs=-1, verbose=1)
            # gs.fit(X_train, y_train)
            # best_model = gs.best_estimator_  # ✅ Get tuned model

            if para:
                rs = RandomizedSearchCV(model, param_distributions=para, n_iter=20, cv=3, n_jobs=-1, verbose=1, random_state=42)
                rs.fit(X_train, y_train)
                best_model = rs.best_estimator_
            else:
                model.fit(X_train, y_train)
                best_model = model

            tuned_models[model_name] = best_model  # ✅ Store tuned model

            y_train_pred = best_model.predict(X_train)
            y_test_pred = best_model.predict(X_test)

            train_model_score = np.sqrt(mean_squared_error(y_train, y_train_pred))
            test_model_score = np.sqrt(mean_squared_error(y_test, y_test_pred))

            report[model_name] = test_model_score

        return report, tuned_models  # ✅ Return both

    
    # def train_model(self, X_train, y_train, X_test, y_test):
    def train_model(self):
        models = {
            "Linear Regression": LinearRegression(), 
            "Ridge Regression": Ridge(), 
            "Decision Tree": DecisionTreeRegressor(), 
            "Random Forest": RandomForestRegressor(verbose=1), 
            "Gradiant Boosting": GradientBoostingRegressor(verbose=1), 
        }

        model_params = {
            "Linear Regression": {},

            "Decision Tree": {
                "max_depth": [None, 5, 10],
                "min_samples_split": [2, 5, 10, 20],
                "min_samples_leaf": [1, 5, 10, 20],
                "max_features": ["sqrt", "log2"],
                "criterion": ["squared_error", "friedman_mse"]
            },

            "Random Forest": {
                "n_estimators": [100, 200],
                "max_depth": [None, 5, 10],
                "min_samples_split": [2, 5],
                "min_samples_leaf": [1, 2, 4],
                "max_features": ["sqrt", "log2"],
                "bootstrap": [True, False]
            },

            "Gradient Boosting": {
                "n_estimators": [100, 200],
                "learning_rate": [0.01, 0.1],
                "max_depth": [3, 5],
                "min_samples_split": [2, 5],
                "min_samples_leaf": [1, 3],
                "max_features": ["sqrt", "log2"],
                "subsample": [0.7, 0.8]
            },

            "Ridge Regression": {
                "alpha": [0.001, 0.01]
            }
        }

        model_report, tuned_models  = self.evaluate_models(X_train=self.X_train, y_train=self.y_train, X_test=self.X_test, y_test=self.y_test, models=models, params=model_params)

        ## To get best model score from dict 
        # best_model_score = max(sorted(model_report.values()))
        best_model_score = min(sorted(model_report.values()))

        ## To get best model name from dict 
        best_model_name = list(model_report.keys())[
            list(model_report.values()).index(best_model_score)
        ]

        # print(best_model_name)
        logging.info(f"Best model found: {best_model_name}")

        # best_model = models[best_model_name] 
        best_model = tuned_models[best_model_name]
        best_params = best_model.get_params()

        y_train_pred = best_model.predict(self.X_train)
        y_test_pred = best_model.predict(self.X_test)
        # classification_test_metric=get_classification_score(y_true=y_test, y_pred=y_test_pred)

        train_mse = mean_squared_error(self.y_train, y_train_pred)
        train_rmse = np.sqrt(train_mse)
        train_r2 = r2_score(self.y_train, y_train_pred)

        train_reg_metrix = RegressionMetrixArtifacts(
             mse=train_mse, 
             rmse=train_rmse, 
             r2_score=train_r2
        )

        test_mse = mean_squared_error(self.y_test, y_test_pred)
        test_rmse = np.sqrt(test_mse)
        test_r2 = r2_score(self.y_test, y_test_pred)

        test_reg_metrix = RegressionMetrixArtifacts(
             mse=test_mse, 
             rmse=test_rmse, 
             r2_score=test_r2
        )

        model_trainer_artifact = ModelTrainerArtifact(
             trained_model=best_model,
             best_model_name=best_model_name, 
             best_model_parameters=best_params, 
             models_report=model_report, 
             train_metric_artifact=train_reg_metrix, 
             test_metric_artifact=test_reg_metrix
             )

        return model_trainer_artifact


import os 
import sys 
import pandas as pd 
import numpy as np 
import logging

from sklearn.linear_model import LogisticRegression
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.svm import SVC
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.ensemble import AdaBoostClassifier

from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score

from auto_machinelearning.ml.classification.entity.artifacts_entity import ClassificationMetricArtifact, ModelTrainerArtifact, DataTransformationArtifacts


class ModelTrainer: 
    def __init__(self, data_transformation_artifacts: DataTransformationArtifacts): 
        self.data_transformation_artifacts = data_transformation_artifacts
        self.X_train = self.data_transformation_artifacts.X_train
        self.X_test = self.data_transformation_artifacts.X_test
        self.y_train = self.data_transformation_artifacts.y_train
        self.y_test = self.data_transformation_artifacts.y_test


    def evaluate_models(self, X_train, y_train, X_test, y_test, models, params): 
        report = {}
        tuned_models = {}  # NEW: to store best estimators

        for i in range(len(list(models))): 
            model_name = list(models.keys())[i]
            model = list(models.values())[i]
            para = params.get(model_name, {})

            # gs = GridSearchCV(model, para, cv=3, n_jobs=-1, verbose=1)
            # gs.fit(X_train, y_train)
            # best_model = gs.best_estimator_  # ✅ Get tuned model

            if para:
                rs = RandomizedSearchCV(model, param_distributions=para, n_iter=20, cv=3, n_jobs=-1, verbose=1, random_state=42)
                rs.fit(X_train, y_train)
                best_model = rs.best_estimator_
            else:
                model.fit(X_train, y_train)
                best_model = model

            tuned_models[model_name] = best_model  # ✅ Store tuned model

            y_train_pred = best_model.predict(X_train)
            y_test_pred = best_model.predict(X_test)

            train_model_score = accuracy_score(y_train, y_train_pred)
            test_model_score = accuracy_score(y_test, y_test_pred)

            report[model_name] = test_model_score

        return report, tuned_models  # ✅ Return both

    
    # def train_model(self, X_train, y_train, X_test, y_test):
    def train_model(self):
        models = {
            "Logistic Regression": LogisticRegression(), 
            "KNN": KNeighborsClassifier(), 
            "Naive Bayes": GaussianNB(), 
            "Decision Tree": DecisionTreeClassifier(), 
            "Random Forest": RandomForestClassifier(verbose=1), 
            "Gradiant Boosting": GradientBoostingClassifier(verbose=1), 
            "ADA Boost": AdaBoostClassifier()
        }

        model_params = {
            "Logistic Regression": {
                'penalty': ['l1', 'l2', 'elasticnet'], 
                'C': np.logspace(-4, 4, 20), 
                'solver': ['liblinear', 'saga'], 
                'max_iter': [100, 200, 500, 1000] 
            },

            "KNN": {
                'n_neighbors': np.arange(1, 10), 
                'weights': ['uniform', 'distance'], 
                'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute'], 
                'p': [1, 2] 
            }, 

            "Naive Bayes": {}, 

            "Decision Tree": {
                'criterion': ['gini', 'entropy'], 
                'max_depth': [5, 10, 15, 20], 
                'min_samples_split': [2, 5, 10], 
                'min_samples_leaf': [1, 2, 4], 
                'max_features': ['sqrt', 'log2'] 
            },

            "Random Forest": {
                'n_estimators': [50, 100, 200], 
                'max_depth': [5, 10, 15], 
                'min_samples_split': [2, 5, 10], 
                'min_samples_leaf': [1, 2, 4], 
                'max_features': ['sqrt', 'log2'], 
                'bootstrap': [True, False] 
            },

            "Gradient Boosting": {
                'n_estimators': [50, 100, 200, 300],
                'learning_rate': [0.01, 0.05, 0.1, 0.2],
                'max_depth': [3, 5, 7], 
                'subsample': [0.7, 0.8, 0.9, 1.0],
                'max_features': ['sqrt', 'log2'] 
            },

            "ADA Boost": {
                'n_estimators': [50, 100, 200, 300],
                'learning_rate': [0.01, 0.05, 0.1, 0.2],
                # 'base_estimator': [DecisionTreeClassifier(max_depth=1), DecisionTreeClassifier(max_depth=2)] # Consider a shallow Decision Tree.
            }
        }

        model_report, tuned_models  = self.evaluate_models(X_train=self.X_train, y_train=self.y_train, X_test=self.X_test, y_test=self.y_test, models=models, params=model_params)

        ## To get best model score from dict 
        # best_model_score = max(sorted(model_report.values()))
        best_model_score = max(sorted(model_report.values()))

        ## To get best model name from dict 
        best_model_name = list(model_report.keys())[
            list(model_report.values()).index(best_model_score)
        ]

        # print(best_model_name)
        logging.info(f"Best model found: {best_model_name}")

        # best_model = models[best_model_name] 
        best_model = tuned_models[best_model_name]
        best_params = best_model.get_params()

        y_train_pred = best_model.predict(self.X_train)
        y_test_pred = best_model.predict(self.X_test)
        # classification_test_metric=get_classification_score(y_true=y_test, y_pred=y_test_pred)

        train_accuracy = accuracy_score(self.y_train, y_train_pred)
        train_precision = precision_score(self.y_train, y_train_pred)
        train_recall = recall_score(self.y_train, y_train_pred)
        train_f1score = f1_score(self.y_train, y_train_pred)

        train_cls_metrix = ClassificationMetricArtifact(
            accuracy=train_accuracy, 
            f1_score=train_f1score, 
            recall_score=train_recall, 
            precision_score=train_precision
        )

        test_accuracy = accuracy_score(self.y_test, y_test_pred)
        test_precision = precision_score(self.y_test, y_test_pred)
        test_recall = recall_score(self.y_test, y_test_pred)
        test_f1score = f1_score(self.y_test, y_test_pred)

        test_cls_metrix = ClassificationMetricArtifact(
            accuracy=test_accuracy, 
            f1_score=test_f1score, 
            recall_score=test_recall, 
            precision_score=test_precision
        )

        model_trainer_artifact = ModelTrainerArtifact(
             trained_model=best_model,
             best_model_name=best_model_name, 
             best_model_parameters=best_params, 
             models_report=model_report, 
             train_metric_artifact=train_cls_metrix, 
             test_metric_artifact=test_cls_metrix
             )

        return model_trainer_artifact


import pandas as pd 
import numpy as np

class DataDescribe:
    def __init__(self, df, target_column=None):
        self.df = df
        self.target_column = target_column

        self.num_rows = df.shape[0]
        self.num_cols = df.shape[1]
        self.column_names = df.columns.tolist()

        self.dtypes = df.dtypes.to_dict()

        self.categorical_columns = df.select_dtypes(include=['object', 'category']).columns.tolist()
        self.numerical_columns = df.select_dtypes(include=['int64', 'float64']).columns.tolist()

        self.num_categorical_columns = len(self.categorical_columns)
        self.num_numerical_columns = len(self.numerical_columns)

        self.missing_values = df.isnull().sum().to_dict()
        self.missing_percentage = (df.isnull().mean() * 100).to_dict()

        self.unique_values = df.nunique().to_dict()

        self.basic_stats = df.describe().to_dict()

        self.correlation_matrix = df.corr(numeric_only=True).to_dict()

        if target_column and target_column in df.columns:
            self.target_info = {
                "type": "classification" if df[target_column].nunique() <= 20 else "regression",
                "distribution": df[target_column].value_counts().to_dict()
            }
        else:
            self.target_info = {
                "type": "unknown"
            }

    def get_summary_dict(self):
        return {
            "num_rows": self.num_rows,
            "num_columns": self.num_cols,
            "column_names": self.column_names,
            "numerical_columns": self.numerical_columns,
            "categorical_columns": self.categorical_columns,
            "num_numerical_columns": self.num_numerical_columns,
            "num_categorical_columns": self.num_categorical_columns,
            "data_types": {k: str(v) for k, v in self.dtypes.items()},
            "missing_values": self.missing_values,
            "missing_percentage": {k: round(v, 2) for k, v in self.missing_percentage.items()},
            "unique_values": self.unique_values,
            "basic_statistics": {k: {col: round(v, 2) for col, v in cols.items()} for k, cols in self.basic_stats.items()},
            "correlation_matrix": {
                col: {k: round(v, 2) for k, v in corr.items()} for col, corr in self.correlation_matrix.items()
            },
            "target_info": self.target_info
        }

    def summarize(self):
        summary = self.get_summary_dict()
        print("📊 DATASET SUMMARY")
        print("="*50)
        print(f"🧾 Rows: {summary['num_rows']}")
        print(f"📐 Columns: {summary['num_columns']}")
        print(f"🔢 Numerical Columns ({summary['num_numerical_columns']}): {summary['numerical_columns']}")
        print(f"🔤 Categorical Columns ({summary['num_categorical_columns']}): {summary['categorical_columns']}")
        print()

        print("🧪 Data Types:")
        for col, dtype in summary['data_types'].items():
            print(f"   - {col}: {dtype}")
        print()

        print("🕳️ Missing Values:")
        missing = summary['missing_values']
        percent = summary['missing_percentage']
        for col in missing:
            if missing[col] > 0:
                print(f"   - {col}: {missing[col]} missing ({percent[col]}%)")
        if all(v == 0 for v in missing.values()):
            print("   - No missing values.")
        print()

        print("🔁 Unique Values:")
        for col, val in summary['unique_values'].items():
            print(f"   - {col}: {val} unique values")
        print()

        print("📈 Basic Statistics (Numerical):")
        for stat, values in summary['basic_statistics'].items():
            print(f"   {stat}:")
            for col, val in values.items():
                print(f"     - {col}: {val}")
        print()

        print("🔗 Correlation Matrix:")
        for col, corr_dict in summary['correlation_matrix'].items():
            print(f"   {col}:")
            for target_col, corr_val in corr_dict.items():
                if col != target_col:
                    print(f"     - {target_col}: {corr_val}")
        print()

        if summary['target_info']:
            print("🎯 Target Info:")
            print(f"   - Type: {summary['target_info']['type']}")
            # print(f"   - Distribution:")
            # for label, count in summary['target_info']['distribution'].items():
            #     print(f"     - {label}: {count}")
        print("="*50)



import os
import joblib
import pandas as pd
from sklearn.pipeline import Pipeline
from sklearn.experimental import enable_iterative_imputer  # ✅ This enables the feature of IterativeImputer
from sklearn.impute import SimpleImputer, IterativeImputer
from sklearn.compose import ColumnTransformer
from sklearn.preprocessing import OrdinalEncoder
from sklearn.linear_model import BayesianRidge


class Preprocess: 
    def __init__(self, df: pd.DataFrame, target_column: str , data_store_path: str, pre_processor_store_path: str): 
        self.df = df 
        self.target_column = target_column
        self.data_store_path = data_store_path
        self.pre_processor_store_path = pre_processor_store_path
        self.preprocessor = None
        self.categorical_columns = None 
        self.numerical_columns = None 

    def fit(self) -> pd.DataFrame: 
        df_copy = self.df.copy()

        if self.target_column: 
            X = df_copy.drop(columns=[self.target_column])
            y = df_copy[self.target_column]

        else: 
            X = df_copy

        # Identify column types
        self.categorical_columns = X.select_dtypes(include=['object', 'category']).columns.tolist()
        self.numerical_columns = X.select_dtypes(include=['int64', 'float64']).columns.tolist()

        # Categorical column 
        cat_pipeline = Pipeline(steps=[
            ('encoder', OrdinalEncoder(handle_unknown='use_encoded_value', unknown_value=-1)),
            ('imputer', SimpleImputer(strategy='most_frequent'))
        ])

        # Pipeline for numerical: just pass (IterativeImputer will handle it)
        num_pipeline = 'passthrough'

        # ColumnTransformer: preprocess categorical only
        preprocessor = ColumnTransformer(transformers=[
            ('cat', cat_pipeline, self.categorical_columns),
            ('num', num_pipeline, self.numerical_columns)
        ])

        # Full pipeline with IterativeImputer
        full_pipeline = Pipeline(steps=[
            ('preprocessor', preprocessor),
            ('imputer', IterativeImputer(estimator=BayesianRidge(), random_state=0))
        ])

        # Transform data
        X_transformed = full_pipeline.fit_transform(X)

        # Convert to DataFrame
        X_transformed = pd.DataFrame(X_transformed, columns=self.categorical_columns + self.numerical_columns)
        final_df = pd.concat([X_transformed.reset_index(drop=True), y.reset_index(drop=True)], axis=1)

        # Save the data and preprocessor object to a file
        data_dir = os.path.dirname(self.data_store_path)
        os.makedirs(data_dir, exist_ok=True)

        processor_dir = os.path.dirname(self.pre_processor_store_path)
        os.makedirs(processor_dir, exist_ok=True)
       
        final_df.to_csv(self.data_store_path, index=False)
        joblib.dump(preprocessor, self.pre_processor_store_path)

        return final_df




import os, sys 
import pandas as pd 
import numpy as np 

from auto_machinelearning.ml.regression.entity.artifacts_entity import DataIngestionArtifacts
from auto_machinelearning.ml.regression.entity.artifacts_entity import DataTransformationArtifacts

class DataTransformation: 
    def __init__(self, data_ingestion_artifacts: DataIngestionArtifacts, target_column: str): 
        self.train_df = data_ingestion_artifacts.train_df.copy()
        self.test_df = data_ingestion_artifacts.test_df.copy()
        self.target_column = target_column

    def run_data_transformation(self) -> DataTransformationArtifacts:
        # Separate X and y
        X_train = self.train_df.drop(columns=[self.target_column])
        y_train = self.train_df[[self.target_column]]
        X_test = self.test_df.drop(columns=[self.target_column])
        y_test = self.test_df[[self.target_column]]

        # Return all in artifacts
        return DataTransformationArtifacts(
            X_train=X_train,
            X_test=X_test,
            y_train=y_train,
            y_test=y_test
        )


import pandas as pd
from sklearn.preprocessing import LabelEncoder, StandardScaler


class AutoPreprocessor:
    def __init__(self, df: pd.DataFrame, target_column: str=None):
        self.df = df
        self.target_column = target_column
        self.encoder = LabelEncoder()
        self.scaler = StandardScaler()


    # def column_removal(self, df: pd.DataFrame) -> pd.DataFrame: 
    #     remove_col_list = []
    #     df_copy = df.copy()
    #     nan_val = df.isna().sum()

    #     for key, value in dict(nan_val).items():
    #         percentage = value/df_copy.shape[0] * 100
    #         if percentage > 30: 
    #             remove_col_list.append(key)

    #     updated_df = df_copy.drop(columns = remove_col_list)

    #     return updated_df
    
    def convert_cat_to_num(self, df: pd.DataFrame) -> pd.DataFrame:
        df_copy = df.copy() 
        categorical_columns = df_copy.select_dtypes(include=['object', 'category']).columns.tolist()

        for col in df.columns: 
            if col in categorical_columns: 
                df_copy[col] = self.encoder.fit_transform(df_copy[col].astype(str))
        
        return df_copy
    
    def scale_dataframe(self, df: pd.DataFrame) -> pd.DataFrame: 
        df_copy = df.copy()
        scale_cols = [col for col in df_copy.columns if col != self.target_column]
        if scale_cols:
            df_copy[scale_cols] = self.scaler.fit_transform(df_copy[scale_cols])

        return df_copy
    
    def transform_new_data(self, new_df: pd.DataFrame) -> pd.DataFrame:
        df_copy = new_df.copy()

        # Use already-fitted encoder and scaler
        df_copy = self.convert_cat_to_num(df_copy)
        df_copy = self.scale_dataframe(df_copy)

        return df_copy
    
    def run_preprocessing(self): 
        # updated_df = self.column_removal(self.df)
        encoded_df = self.convert_cat_to_num(self.df)
        scaled_df = self.scale_dataframe(encoded_df)

        return scaled_df, self.encoder, self.scaler




from setuptools import find_packages , setup
from typing import List

HYPEN_E_DOT ="-e ."


def get_requirements(file_path:str)->List[str]:
    requirements=[]
    with open(file_path) as file_obj:
        requirements=file_obj.readlines()
        requirements=[req.replace("\n","") for req in requirements]

        if HYPEN_E_DOT in requirements:
            requirements.remove(HYPEN_E_DOT)
        return requirements

setup(
    name='ML-Genius',
    version='0.0.1',
    author='PratikRathod',
    author_email='pratikr1521998@gmail.com',
    description="An AutoML framework with classification, regression, etc.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.12",
    install_requires=get_requirements("requirements.txt"),
)


import pandas as pd 
from dataclasses import dataclass 

@dataclass
class DataIngestionArtifacts:
    df: pd.DataFrame
    train_df: pd.DataFrame 
    test_df: pd.DataFrame

@dataclass
class DataTransformationArtifacts: 
    X_train: pd.DataFrame
    X_test: pd.DataFrame
    y_train: pd.DataFrame
    y_test:pd.DataFrame

@dataclass
class RegressionMetrixArtifacts: 
    mse: str 
    rmse: str 
    r2_score: str

@dataclass 
class ClassificationMetricArtifact: 
    accuracy: str
    f1_score: str 
    precision_score: str 
    recall_score: str 

@dataclass 
class ModelTrainerArtifact: 
    trained_model: object 
    best_model_name: str
    best_model_parameters: dict
    models_report: dict
    train_metric_artifact: ClassificationMetricArtifact | RegressionMetrixArtifacts
    test_metric_artifact: ClassificationMetricArtifact | RegressionMetrixArtifacts




import pandas as pd 


class ReadData:
    def __init__(self): 
        self.df = None 

    def read_csv(self, csv_path: str, **params) -> pd.DataFrame:
        self.df = pd.read_csv(csv_path, **params)
        return self.df
    
    def read_excel(self, excel_path: str, **params) -> pd.DataFrame: 
        self.df = pd.read_excel(excel_path, **params)
        return self.df